package net.wanji.opt.synthesis.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.Constants;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.OkHttpClientUtil;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.dao.entity.GreenwaveInfoPO;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.dao.mapper.CrossDataRealtimeMapper;
import net.wanji.databus.dao.mapper.GreenwaveInfoMapper;
import net.wanji.databus.entity.basedata.ResultView;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.databus.vo.AbnormalCrossListVO;
import net.wanji.opt.cache.BaseCrossInfoCache;
import net.wanji.opt.cache.GreenWaveInfoCache;
import net.wanji.opt.common.enums.EventStatusEnum;
import net.wanji.opt.common.enums.StrategyControlEnum;
import net.wanji.opt.dao.mapper.*;
import net.wanji.opt.dto.StrategyOptCountDTO;
import net.wanji.opt.dto.StrategyOptTimesDTO;
import net.wanji.opt.po.StrategyGreenOptHistEntity;
import net.wanji.opt.po.trend.HoloEventInfoPO;
import net.wanji.opt.synthesis.enums.StrategyCrossAlgoEnum;
import net.wanji.opt.synthesis.pojo.*;
import net.wanji.opt.synthesis.pojo.vo.*;
import net.wanji.opt.synthesis.service.PushStrategyControlService;
import net.wanji.opt.synthesis.service.StrategyControlService;
import org.apache.commons.lang3.StringUtils;
import org.geotools.feature.visitor.CountVisitor;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/11/03 17:37
 */
@Slf4j
@Service
public class StrategyControlServiceImpl implements StrategyControlService {

    @Resource
    private PushStrategyControlService pushStrategyControlService;
    @Resource
    private StrategyControlInfoMapper strategyControlInfoMapper;
    @Resource
    private StrategyPlanInfoMapper strategyPlanInfoMapper;
    @Resource
    private StrategyDailyPlanInfoMapper strategyDailyPlanInfoMapper;
    @Resource
    private BaseCrossInfoMapper baseCrossInfoMapper;
    @Resource
    private StrategyFactoryMapper strategyFactoryMapper;
    @Resource
    private GreenwaveInfoMapper greenwaveInfoMapper;
    @Resource
    private StrategyGreenOptHistMapper strategyGreenOptHistMapper;
    @Resource
    private StrategyCrossResultMapper strategyCrossResultMapper;
    @Resource
    private BaseCrossInfoCache baseCrossInfoCache;
    @Resource
    private CrossDataRealtimeMapper crossDataRealtimeMapper;
    @Resource
    private HoloEventMapper holoEventMapper;


    @Override
    public JsonViewObject strategyInfoOperation(StrategyControlVO strategyControlVO) throws Exception {

        try {
            // 存库
            insertAndUpdate(strategyControlVO);

            StrategyControlReq req = convertReq(strategyControlVO);
            Result result = pushStrategyControlService.push(req);
            if (Objects.nonNull(result)) {
                JsonViewObject jsonViewObject = JsonViewObject.newInstance();
                jsonViewObject.setCode(Integer.valueOf(result.getCode()));
                jsonViewObject.setMessage(result.getMsg());
                jsonViewObject.setContent(result.getData());
                jsonViewObject.setStatus(StringUtils.equals("200", result.getCode()) ? Constants.JsonView.STATUS_SUCCESS : Constants.JsonView.STATUS_FAIL);
                return jsonViewObject;
            } else {
                return JsonViewObject.newInstance().fail("推送神思策略控制失败！");
            }
        } catch (Exception e) {
            log.error("策略操作失败：{}", e);
            throw new Exception(e);
        }
    }

    private void insertAndUpdate(StrategyControlVO strategyControlVO) throws IllegalAccessException, InvocationTargetException, JsonProcessingException {
        List<StrategyControlDataVO> dataList = strategyControlVO.getDataList();
        for (StrategyControlDataVO dataVO : dataList) {
            StrategyControlDataEntity entity = new StrategyControlDataEntity();
            BeanUtils.copyProperties(dataVO, entity);
            entity.setTime(JacksonUtils.getInstance().writeValueAsString(dataVO.getTime()));
            if (StringUtils.equals("insert", dataVO.getAction())) {
                entity.setId(null);
                strategyControlInfoMapper.insert(entity);
            } else if (StringUtils.equals("update", dataVO.getAction())) {
                strategyControlInfoMapper.updateById(entity);
            } else if (StringUtils.equals("delete", dataVO.getAction())) {
                strategyControlInfoMapper.deleteById(entity);
            }
        }
    }

    private StrategyControlReq convertReq(StrategyControlVO vo) throws Exception {
        try {
            List<StrategyControlDataVO> dataList = vo.getDataList();
            StrategyControlReq req = new StrategyControlReq();
            if (!CollectionUtils.isEmpty(dataList)) {
                List<StrategyControlDataReq> dataReqList = new ArrayList<>(dataList.size());
                for (StrategyControlDataVO dataVO : dataList) {
                    StrategyControlDataReq detailData = new StrategyControlDataReq();
                    detailData.setBiz_id(dataVO.getBizId());
                    detailData.setBiz_type(dataVO.getBizType());
                    detailData.setStrategy(dataVO.getStrategy());
                    if (Objects.nonNull(dataVO.getScheduleStart()) && Objects.nonNull(dataVO.getScheduleStart())) {
                        detailData.setSchedule_start(DateUtil.format(dataVO.getScheduleStart(), Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND));
                        detailData.setSchedule_end(DateUtil.format(dataVO.getScheduleEnd(), Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND));
                    } else {
                        detailData.setSchedule_start(DateUtil.format(new Date(), Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND));
                        detailData.setSchedule_end(DateUtil.format(new Date(), Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND));
                    }
                    List<StrategyControlDataVO.TimeTable> timeTableList = dataVO.getTime();
                    List<StrategyControlDataReq.Time_table> timeTables = new ArrayList<>(timeTableList.size());
                    timeTableList.forEach(item -> {
                        StrategyControlDataReq.Time_table timeTable = new StrategyControlDataReq.Time_table();
                        timeTable.setTime_list(item.getTimeList());
                        timeTable.setWeek(item.getWeek());
                        timeTables.add(timeTable);
                    });
                    detailData.setTime_table(timeTables);
                    detailData.setFreq(dataVO.getFrequency());
                    detailData.setStatus(dataVO.getStatus());
                    detailData.setAction(StringUtils.equals("update", dataVO.getAction()) ? "insert" : dataVO.getAction());

                    dataReqList.add(detailData);
                    req.setData(dataReqList);
                }
            }
            return req;
        } catch (Exception e) {
            log.error("策略控制操作数据转换异常：{}", e);
            throw new Exception(e);
        }
    }

    @Override
    public JsonViewObject strategyInfoPageList(StrategyControlDataEntity entity) throws Exception {
        LambdaQueryWrapper<StrategyControlDataEntity> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotBlank(entity.getBizId())) {
            queryWrapper.eq(StrategyControlDataEntity::getBizId, entity.getBizId());
        }
        if (Objects.nonNull(entity.getBizType())) {
            queryWrapper.eq(StrategyControlDataEntity::getBizType, entity.getBizType());
        }
        if (Objects.nonNull(entity.getStrategy())) {
            queryWrapper.eq(StrategyControlDataEntity::getStrategy, entity.getStrategy());
        }
        if (Objects.nonNull(entity.getStatus())) {
            queryWrapper.eq(StrategyControlDataEntity::getStatus, entity.getStatus());
        }
        Page<StrategyControlDataEntity> page = new Page<>(entity.getPageNum(), entity.getPageSize());
        Page<StrategyControlDataEntity> pageInfo = strategyControlInfoMapper.selectPage(page, queryWrapper);
        return JsonViewObject.newInstance().success(pageInfo.getRecords());
    }

    @Override
    public JsonViewObject crossStrategyHistList(Integer strategyType, Integer groupType, Integer resultType) throws Exception {
        List<StrategyControlHistVO> results = new ArrayList<>();
        try {
            if (Objects.isNull(groupType)) {
                groupType = 1;
            }
            LocalDateTime localDateTime = getLocalDateTime(groupType);

            // 查询干线和路口
            if (Objects.isNull(strategyType)) {
                setGreenOptHist(results, localDateTime, resultType);
                setCrossOptHist(results, localDateTime, resultType);
            } // 路口
            if (Objects.equals(0, strategyType)) {
                setCrossOptHist(results, localDateTime, resultType);
            }
            // 绿波历史记录
            if (Objects.equals(1, strategyType)) {
                setGreenOptHist(results, localDateTime, resultType);
            }
        } catch (Exception e) {
            log.error("优化策略查询失败：", e);
            JsonViewObject.newInstance().fail("优化策略查询失败");
        }
        List<StrategyControlHistVO> sorted = results.stream().sorted(Comparator.comparing(StrategyControlHistVO::getOptTime).reversed()).collect(Collectors.toList());
        return JsonViewObject.newInstance().success(sorted);
    }

    @NotNull
    private static LocalDateTime getLocalDateTime(Integer groupType) {
        LocalDateTime localDateTime = null;
        switch (groupType) {
            case 0:
                localDateTime = DateUtil.getPlusHour(-1);
                break;
            case 1:
                localDateTime = DateUtil.getMidNight();
                break;
            case 2:
                localDateTime = DateUtil.getPlusSecond(-5);
                break;
            default:
                localDateTime = DateUtil.getMidNight();
        }
        return localDateTime;
    }

    private void setCrossOptHist(List<StrategyControlHistVO> results, LocalDateTime midnight, Integer resultType) {
        LambdaQueryWrapper<StrategyCrossResultEntity> queryWrapper = new LambdaQueryWrapper<>();
        if (Objects.nonNull(midnight)) {
            queryWrapper.ge(StrategyCrossResultEntity::getIssueTime, midnight);
        }
        if (Objects.nonNull(resultType)) {
            if (Objects.equals(1, resultType)) {
                queryWrapper.eq(StrategyCrossResultEntity::getResponseCode, 200);
            } else {
                queryWrapper.ne(StrategyCrossResultEntity::getResponseCode, 200);
            }
        }
        List<StrategyCrossResultEntity> resultEntities = strategyCrossResultMapper.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(resultEntities)) {
            for (StrategyCrossResultEntity resultEntity : resultEntities) {
                StrategyControlHistVO histVO = new StrategyControlHistVO();
                String crossId = resultEntity.getCrossId();
                histVO.setId(crossId);
                histVO.setName(resultEntity.getCrossName());
                String coordinateByCrossId = BaseCrossInfoCache.getCoordinateByCrossId(crossId);
                if (StringUtils.isBlank(coordinateByCrossId)) {
                    continue;
                }
                histVO.setWkt(coordinateByCrossId.replace("POINT(", "").replace(" ", ",").replace(")", ""));
                histVO.setStrategy(resultEntity.getCurrentAlgo());
                histVO.setStrategyName(StrategyCrossAlgoEnum.getDescByCode(resultEntity.getCurrentAlgo()));
                Date issueTime = resultEntity.getIssueTime();
                String format = DateUtil.format(issueTime, Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND);
                histVO.setOptTime(format);
                //if (resultEntity.getCurrentAlgo() != 1) {
                //    histVO.setResult(Objects.equals(200, resultEntity.getResponseCode()) ? "成功" : "失败");
                //} else {
                //    histVO.setResult("成功");
                //}
                histVO.setResult(resultEntity.getResponseContent());
                histVO.setCode(resultEntity.getResponseCode());
                results.add(histVO);
            }
        }
    }

    private void setGreenOptHist(List<StrategyControlHistVO> results, LocalDateTime midnight, Integer resultType) throws ParseException {
        LambdaQueryWrapper<StrategyGreenOptHistEntity> queryWrapper = new LambdaQueryWrapper<>();
        if (Objects.nonNull(midnight)) {
            queryWrapper.ge(StrategyGreenOptHistEntity::getControlTime, midnight);
        }
        if (Objects.nonNull(resultType)) {
            if (Objects.equals(1, resultType)) {
                queryWrapper.eq(StrategyGreenOptHistEntity::getResponseCode, 200);
            } else {
                queryWrapper.ne(StrategyGreenOptHistEntity::getResponseCode, 200);
            }
        }
        List<StrategyGreenOptHistEntity> entities = strategyGreenOptHistMapper.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(entities)) {
            Map<Integer, GreenwaveInfoPO> greenWaveMap = GreenWaveInfoCache.greenWaveMap;
            for (StrategyGreenOptHistEntity entity : entities) {
                StrategyControlHistVO histVO = new StrategyControlHistVO();
                histVO.setId(String.valueOf(entity.getGreenId()));
                if (greenWaveMap.isEmpty()) {
                    continue;
                }
                String controlTime = entity.getControlTime();
                Date parse = DateUtil.parse(controlTime, "yyyy-MM-dd HH:mm:ss");
                GreenwaveInfoPO greenwaveInfoPO = greenWaveMap.get(entity.getGreenId());
                if (Objects.nonNull(greenwaveInfoPO)) {
                    histVO.setName(greenwaveInfoPO.getName());
                    histVO.setWkt(greenwaveInfoPO.getWkt());
                }
                LambdaQueryWrapper<StrategyControlDataEntity> strategyQuery = new LambdaQueryWrapper<>();
                strategyQuery.eq(StrategyControlDataEntity::getBizId, entity.getGreenId());
                strategyQuery.eq(StrategyControlDataEntity::getBizType, 1);
                strategyQuery.le(StrategyControlDataEntity::getScheduleStart, parse);
                strategyQuery.ge(StrategyControlDataEntity::getScheduleEnd, parse);
                // 应该是下发时间在调度下发时间内，但是目前神思下发可能没有走策略
                List<StrategyControlDataEntity> strategyList = strategyControlInfoMapper.selectList(strategyQuery);
                if (!CollectionUtils.isEmpty(strategyList)) {
                    StrategyControlDataEntity strategyControlDataEntity = strategyList.get(0);
                    histVO.setStrategy(strategyControlDataEntity.getStrategy());
                    histVO.setStrategyName("绿波带");
                } else {
                    histVO.setStrategy(0);
                    histVO.setStrategyName("绿波带");
                }
                histVO.setOptTime(entity.getControlTime());
                //histVO.setResult(Objects.equals(1, entity.getControlMethod()) ? "失败" : "成功");
                histVO.setResult(entity.getResponseContent());
                histVO.setCode(entity.getResponseCode());
                results.add(histVO);
            }
        }
    }

    @Override
    public JsonViewObject crossStrategyInfoList(Integer type) throws Exception {
        try {
            List<StrategyControlDataEntity> results = new ArrayList<>();
            List<StrategyControlDataEntity> entities = getCurrentStrateInfoList(type);
            List<StrategyControlDataEntity> runList = entities.stream().filter(entity -> Objects.equals(1, entity.getStatus())).collect(Collectors.toList());
            Map<String, StrategyControlDataEntity> map = runList.stream().collect(Collectors.toMap(StrategyControlDataEntity::getBizId, entity -> entity));
            // 路口
            if (Objects.equals(0, type)) {
                List<AbnormalCrossListVO> abnormalCrossListVOS = crossDataRealtimeMapper.selectAbnormalCross(null, null, null);
                if (!CollectionUtils.isEmpty(abnormalCrossListVOS)) {
                    for (AbnormalCrossListVO baseCrossInfoPO : abnormalCrossListVOS) {
                        StrategyControlDataEntity strategyControlDataEntity = new StrategyControlDataEntity();
                        String crossId = baseCrossInfoPO.getId();
                        strategyControlDataEntity.setBizType(0);
                        strategyControlDataEntity.setBizId(crossId);
                        // 转化成 0 畅通 1失衡 2拥堵 3溢出 4死锁 5空放 6干线拥堵 7干线缓行 8干线畅通
                        Integer status = baseCrossInfoPO.getRealtimeStatus();
                        strategyControlDataEntity.setStrategy(status);
                        strategyControlDataEntity.setOptMethod(Objects.equals(1, status) ? "均衡调控" : "效率提升");
                        strategyControlDataEntity.setTime(null);
                        strategyControlDataEntity.setStatus(status);
                        strategyControlDataEntity.setCrossName(baseCrossInfoPO.getName());
                        String location = baseCrossInfoPO.getLocationStr();
                        location = location.replace("POINT(", "").replace(" ", ",").replace(")", "");
                        strategyControlDataEntity.setWkt(location);
                        results.add(strategyControlDataEntity);
                    }
                }
            } // 干线
            if (Objects.equals(1, type)) {
                List<GreenwaveInfoPO> greenwaveInfoPOS = greenwaveInfoMapper.selectAll();
                if (!CollectionUtils.isEmpty(greenwaveInfoPOS)) {
                    for (GreenwaveInfoPO greenwaveInfoPO : greenwaveInfoPOS) {
                        StrategyControlDataEntity strategyControlDataEntity = new StrategyControlDataEntity();
                        String greenId = String.valueOf(greenwaveInfoPO.getId());
                        strategyControlDataEntity.setBizType(1);
                        strategyControlDataEntity.setBizId(String.valueOf(greenId));
                        strategyControlDataEntity.setStrategy(8);
                        strategyControlDataEntity.setTime(null);
                        strategyControlDataEntity.setStatus(0);
                        if (map.containsKey(greenId)) {
                            strategyControlDataEntity = map.get(greenId);
                        }
                        strategyControlDataEntity.setCrossName(greenwaveInfoPO.getName());
                        strategyControlDataEntity.setWkt(greenwaveInfoPO.getWkt());
                        strategyControlDataEntity.setOptMethod("效率提升");
                        results.add(strategyControlDataEntity);
                    }
                }
            }
            return JsonViewObject.newInstance().success(results);
        } catch (Exception e) {
            log.error("策略控制查询列表：{}", e);
            throw new RuntimeException(e);
        }
    }

    @Override
    public JsonViewObject strategyOptTimes() throws Exception {
        String timeUrl = "http://37.12.182.29:15020/decisionPage/MonitorStrategyOptimizationInfo/getData";
        String timeResult = OkHttpClientUtil.get(timeUrl);
        ObjectMapper mapper = JacksonUtils.getInstance();
        StrategyOptTimesVO strategyOptTimesVO = new StrategyOptTimesVO();
        if (StringUtils.isNotBlank(timeResult)) {
            StrategyOptTimesDTO dto = mapper.readValue(timeResult, StrategyOptTimesDTO.class);
            List<StrategyOptTimesDTO.Detail> content = dto.getContent();
            if (!CollectionUtils.isEmpty(content)) {
                Double totalTime = 0.0;
                for (StrategyOptTimesDTO.Detail item : content) {
                    totalTime += item.getOptimizeTime();
                }
                Double temp = Math.floor(totalTime / 3600);
                strategyOptTimesVO.setTimes(temp.intValue());
            }
        }
        String countUrl = "http://37.12.182.29:15020/decisionPage/MonitorRadarIndicatorsInfo/getRadarCountData";
        String countResult = OkHttpClientUtil.get(countUrl);
        if (StringUtils.isNotBlank(countResult)) {
            StrategyOptCountDTO strategyOptCountDTO = mapper.readValue(countResult, StrategyOptCountDTO.class);
            StrategyOptCountDTO.Detail content = strategyOptCountDTO.getContent();
            int count = content.getCongestionNum() + content.getEmptyPassNum() + content.getSpilloverNum() + content.getUnbalanceNum();
            strategyOptTimesVO.setCount(count);
        }
        return JsonViewObject.newInstance().success(strategyOptTimesVO);
    }

    private List<StrategyControlDataEntity> getCurrentStrateInfoList(Integer type) throws Exception {
        ObjectMapper instance = JacksonUtils.getInstance();
        LambdaQueryWrapper<StrategyControlDataEntity> queryWrapper = new LambdaQueryWrapper<>();
        Date current = new Date();
        queryWrapper.eq(StrategyControlDataEntity::getBizType, type);
        queryWrapper.le(StrategyControlDataEntity::getScheduleStart, current);
        queryWrapper.ge(StrategyControlDataEntity::getScheduleEnd, current);
        List<StrategyControlDataEntity> entities = strategyControlInfoMapper.selectList(queryWrapper);
        List<StrategyControlDataEntity> results = new ArrayList<>(entities.size());
        for (StrategyControlDataEntity entity : entities) {
            StrategyControlDataEntity result = new StrategyControlDataEntity();
            BeanUtils.copyProperties(entity, result);
            String time = entity.getTime();
            List<StrategyControlDataVO.TimeTable> timeTables = instance.readValue(time, new TypeReference<List<StrategyControlDataVO.TimeTable>>() {
            });
            for (StrategyControlDataVO.TimeTable timeTable : timeTables) {
                String[] timeList = timeTable.getTimeList();
                for (String s : timeList) {
                    String[] hours = s.split(",");
                    for (String hour : hours) {
                        String[] currentHour = hour.split("-");
                        String startHour = currentHour[0];
                        String entHour = currentHour[1];
                        if (StringUtils.equals("24:00", entHour)) {
                            entHour = "23:59";
                        }
                        String format = DateUtil.format(current, Constants.DATE_FORMAT.E_DATE_FORMAT_HOUR_MINUTE);
                        Date currentTime = DateUtil.parse(format, "HH:mm");
                        Date startHourDate = DateUtil.parse(startHour, "HH:mm");
                        Date endHourDate = DateUtil.parse(entHour, "HH:mm");
                        if (currentTime.after(startHourDate) && currentTime.before(endHourDate)) {
                            result.setTime(hour);
                            result.setStatus(1);
                        } else {
                            // 如果有调度策略在执行，设置为开启，否则关闭，提供前端过滤配置和未配置
                            result.setStatus(0);
                            result.setTime(null);
                        }
                    }
                }
            }
            if (type == 1) {
                Map<Integer, GreenwaveInfoPO> greenWaveMap = GreenWaveInfoCache.greenWaveMap;
                if (!greenWaveMap.isEmpty()) {
                    GreenwaveInfoPO greenwaveInfoPO = greenWaveMap.get(Integer.valueOf(entity.getBizId()));
                    result.setCrossName(greenwaveInfoPO.getName());
                    result.setWkt(greenwaveInfoPO.getWkt());
                }
            }
            result.setOptMethod(StrategyControlEnum.getMethod(result.getStrategy()));
            results.add(result);
        }
        return results;
    }

    @Override
    public JsonViewObject crossOptInfoList(Integer type) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            LocalDate currentDate = LocalDate.now();
            LocalTime startTime = LocalTime.MIDNIGHT;
            LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);
            Date date = Date.from(startOfDay.atZone(ZoneId.systemDefault()).toInstant());
            List<StrategyControlDataExt> results = new ArrayList<>();
            if (Objects.equals(0, type)) {
                results = getCrossList(date);
            }
            if (Objects.equals(1, type)) {
                results = getGreenList(date);
            }

            return jsonViewObject.success(results, "路网优化监测查询成功");
        } catch (Exception e) {
            log.error("AI路口查询异常：", e);
            throw new RuntimeException(e);
        }
    }

    private List<StrategyControlDataExt> getGreenList(Date date) {
        List<StrategyControlDataExt> results = new ArrayList<>();
        List<HoloEventInfoPO> holoEventInfoPOS = holoEventMapper.selectAIGreenList(date);
        if (!CollectionUtils.isEmpty(holoEventInfoPOS)) {
            for (HoloEventInfoPO holoEventInfoPO : holoEventInfoPOS) {
                StrategyControlDataExt ext = new StrategyControlDataExt();
                ext.setBizId(holoEventInfoPO.getCrossId());
                ext.setCrossName(holoEventInfoPO.getRemark());
                String location = holoEventInfoPO.getExtend();
                ext.setWkt(location);
                Integer alarmStatus = holoEventInfoPO.getAlarmStatus();
                String optMethod = "效率提升";
                if (Objects.isNull(alarmStatus)) {
                    ext.setStatus(-1);
                } else {
                    ext.setStatus(alarmStatus);
                }
                ext.setOptMethod(optMethod);
                String desc = EventStatusEnum.getDesc(alarmStatus);
                ext.setOptStatus(desc);
                String type = holoEventInfoPO.getType();
                if (StringUtils.isBlank(type)) {
                    ext.setStrategyName("动态绿波");
                }
                if (StringUtils.equals("705", type)) {
                    ext.setStrategyName("干线-缓行");
                }
                if (StringUtils.equals("706", type)) {
                    ext.setStrategyName("干线-拥堵");
                }
                results.add(ext);
            }
        }
        results.sort(Comparator.comparing(StrategyControlDataExt::getStatus).reversed());
        return results;
    }

    private List<StrategyControlDataExt> getCrossList(Date date) {
        List<StrategyControlDataExt> results = new ArrayList<>();
        List<StrategyCrossResultEntity> resultList = strategyCrossResultMapper.selectAICrossList(null);
        if (!CollectionUtils.isEmpty(resultList)) {
            for (StrategyCrossResultEntity entity : resultList) {
                StrategyControlDataExt ext = new StrategyControlDataExt();
                ext.setBizId(entity.getCrossId());
                ext.setCrossName(entity.getCrossName());
                String location = entity.getResponseContent();
                location = location.replace("POINT(", "").replace(" ", ",").replace(")", "");
                ext.setWkt(location);
                Integer currentAlgo = entity.getCurrentAlgo();
                Date issueTime = entity.getIssueTime();
                Date current = new Date();

                // 优化时间大于5分钟，默认优化完
                if (Objects.nonNull(issueTime) && current.getTime() - issueTime.getTime() > 5 * 60 * 1000) {
                    currentAlgo = null;
                }

                if (Objects.isNull(currentAlgo)) {
                    ext.setOptStatus("正常");
                    ext.setOptMethod("畅通");
                } else if (Objects.equals(1, currentAlgo)) {
                    ext.setOptStatus("优化中");
                    ext.setOptMethod("均衡调控");
                } else {
                    ext.setOptStatus("优化中");
                    ext.setOptMethod("效率提升");
                }
                results.add(ext);
            }
        }
        return results;
    }

    //@Override
    public JsonViewObject crossOptInfoList1241(Integer type) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            List<StrategyControlDataEntity> currentStrateInfoList = getCurrentStrateInfoList(type);
            List<StrategyControlDataExt> strategyControlDataExts = new ArrayList<>();
            List<StrategyControlDataExt> results = new ArrayList<>();
            for (StrategyControlDataEntity strategyControlDataEntity : currentStrateInfoList) {
                Integer strategy = strategyControlDataEntity.getStrategy();
                StrategyControlDataExt strategyControlDataExt = new StrategyControlDataExt();
                BeanUtils.copyProperties(strategyControlDataEntity, strategyControlDataExt);
                strategyControlDataExt.setStrategyName(StrategyControlEnum.getDesc(strategy));
                strategyControlDataExt.setOptMethod(StrategyControlEnum.getMethod(strategy));
                strategyControlDataExt.setOptStatus("未优化");
                if (StringUtils.isNotBlank(strategyControlDataEntity.getTime())) {
                    strategyControlDataExt.setOptStatus("优化中");
                    strategyControlDataExts.add(strategyControlDataExt);
                }
            }
            // 路口无优化模式
            if (Objects.equals(0, type)) {
                List<BaseCrossInfoPO> crossInfoCache = baseCrossInfoCache.getCrossInfoCache();
                if (!CollectionUtils.isEmpty(crossInfoCache)) {
                    for (BaseCrossInfoPO baseCrossInfoPO : crossInfoCache) {
                        StrategyControlDataExt ext = new StrategyControlDataExt();
                        ext.setBizId(baseCrossInfoPO.getId());
                        ext.setStrategyName("无策略");
                        ext.setOptStatus("未优化");
                        ext.setOptMethod("畅通");
                        ext.setStatus(0);
                        for (StrategyControlDataExt strategyControlDataExt : strategyControlDataExts) {
                            if (StringUtils.equals(baseCrossInfoPO.getId(), strategyControlDataExt.getBizId())) {
                                ext = strategyControlDataExt;
                            }
                        }
                        String location = baseCrossInfoPO.getLocation();
                        location = location.replace("POINT(", "").replace(" ", ",").replace(")", "");
                        ext.setWkt(location);
                        ext.setCrossName(baseCrossInfoPO.getName());
                        results.add(ext);
                    }
                }
            }
            // 干线无优化模式
            if (Objects.equals(1, type)) {
                if (CollectionUtils.isEmpty(strategyControlDataExts)) {
                    Map<Integer, GreenwaveInfoPO> greenWaveMap = GreenWaveInfoCache.greenWaveMap;
                    for (Map.Entry<Integer, GreenwaveInfoPO> entry : greenWaveMap.entrySet()) {
                        StrategyControlDataExt ext = new StrategyControlDataExt();
                        ext.setStrategy(0);
                        ext.setStrategyName("无策略");
                        ext.setOptStatus("未优化");
                        ext.setOptMethod("效率提升");
                        ext.setStatus(0);
                        ext.setWkt(entry.getValue().getWkt());
                        ext.setCrossName(entry.getValue().getName());
                        ext.setBizId(String.valueOf(entry.getKey()));
                        results.add(ext);
                    }
                } else {
                    results.addAll(strategyControlDataExts);
                }
            }
            results.sort(Comparator.comparing(StrategyControlDataExt::getOptStatus));
            return jsonViewObject.success(results, "路网优化监测查询成功");
        } catch (Exception e) {
            log.error("路网优化监测查询失败: {}", e);
            return jsonViewObject.fail("路网优化监测查询失败");
        }
    }

    @Override
    public JsonViewObject strategyPlanDetail(String crossId) throws Exception {
        try {
            ObjectMapper instance = JacksonUtils.getInstance();
            StrategyControlDetailList result = new StrategyControlDetailList();
            LambdaQueryWrapper<StrategyPlanInfoEntity> planQueryWrapper = new LambdaQueryWrapper<>();
            planQueryWrapper.eq(StrategyPlanInfoEntity::getCrossId, crossId);
            List<StrategyPlanInfoEntity> planInfoEntities = strategyPlanInfoMapper.selectList(planQueryWrapper);
            List<StrategyControlDetailList.ExecutePlan> executePlans = new ArrayList<>();
            for (StrategyPlanInfoEntity planInfoEntity : planInfoEntities) {
                StrategyControlDetailList.ExecutePlan executePlan = new StrategyControlDetailList.ExecutePlan();
                executePlan.setCompany(planInfoEntity.getCompany());
                executePlan.setCrossId(planInfoEntity.getCrossId());
                executePlan.setType(planInfoEntity.getType());
                executePlan.setPlanId(planInfoEntity.getPlanId());
                executePlan.setStartTime(planInfoEntity.getStartTime());
                executePlan.setEndTime(planInfoEntity.getEndTime());
                executePlan.setStatus(planInfoEntity.getStatus());
                executePlan.setType(planInfoEntity.getType());
                String planDetailsStr = planInfoEntity.getPlanDetails();
                List<StrategyControlDetailList.ExecutePlan.PlanDetail> planDetails = instance.readValue(planDetailsStr, new TypeReference<List<StrategyControlDetailList.ExecutePlan.PlanDetail>>() {
                });
                executePlan.setPlanDetails(planDetails);
                executePlans.add(executePlan);
            }
            result.setPlans(executePlans);

            LambdaQueryWrapper<StrategyDailyPlanInfoEntity> dailyPlanQueryWrapper = new LambdaQueryWrapper<>();
            dailyPlanQueryWrapper.eq(StrategyDailyPlanInfoEntity::getCrossId, crossId);
            List<StrategyDailyPlanInfoEntity> dailyPlanInfoEntities = strategyDailyPlanInfoMapper.selectList(dailyPlanQueryWrapper);
            List<StrategyControlDetailList.DailyPlan> dailyPlans = new ArrayList<>();
            for (StrategyDailyPlanInfoEntity dailyPlanInfoEntity : dailyPlanInfoEntities) {
                StrategyControlDetailList.DailyPlan dailyPlan = new StrategyControlDetailList.DailyPlan();
                dailyPlan.setCrossId(dailyPlanInfoEntity.getCrossId());
                dailyPlan.setDailyPlanId(dailyPlanInfoEntity.getDailyPlanId());
                String dailyPlanDetailsStr = dailyPlanInfoEntity.getDailyPlanDetails();
                List<StrategyControlDetailList.DailyPlan.DailyPlanDetail> dailyPlanDetails = instance.readValue(dailyPlanDetailsStr, new TypeReference<List<StrategyControlDetailList.DailyPlan.DailyPlanDetail>>() {
                });
                dailyPlan.setDailyPlanDetails(dailyPlanDetails);
                dailyPlans.add(dailyPlan);
            }
            result.setDailyPlans(dailyPlans);
            result.setCrossId(crossId);
            return JsonViewObject.newInstance().success(result);
        } catch (Exception e) {
            log.error("策略管理查询失败：{}", e);
            throw new Exception(e);
        }
    }

    @Override
    public JsonViewObject strategyPlanSave(StrategyControlDetailList list) throws Exception {
        try {
            if (Objects.nonNull(list)) {
                ObjectMapper mapper = JacksonUtils.getInstance();
                List<StrategyControlDetailList.ExecutePlan> plans = list.getPlans();
                if (!CollectionUtils.isEmpty(plans)) {
                    for (StrategyControlDetailList.ExecutePlan plan : plans) {
                        StrategyPlanInfoEntity planInfoEntity = new StrategyPlanInfoEntity();
                        planInfoEntity.setPlanId(plan.getPlanId());
                        planInfoEntity.setCrossId(plan.getCrossId());
                        planInfoEntity.setType(plan.getType());
                        planInfoEntity.setStartTime(plan.getStartTime());
                        planInfoEntity.setEndTime(plan.getEndTime());
                        planInfoEntity.setCompany(plan.getCompany());
                        planInfoEntity.setPlanDetails(mapper.writeValueAsString(plan.getPlanDetails()));
                        LambdaQueryWrapper<StrategyPlanInfoEntity> planDel = new LambdaQueryWrapper<>();
                        planDel.eq(StrategyPlanInfoEntity::getCrossId, plan.getCrossId());
                        planDel.eq(StrategyPlanInfoEntity::getPlanId, plan.getPlanId());
                        strategyPlanInfoMapper.delete(planDel);
                        strategyPlanInfoMapper.insert(planInfoEntity);
                    }
                }

                List<StrategyControlDetailList.DailyPlan> dailyPlans = list.getDailyPlans();
                for (StrategyControlDetailList.DailyPlan dailyPlan : dailyPlans) {
                    Integer dailyPlanId = dailyPlan.getDailyPlanId();
                    String crossId = dailyPlan.getCrossId();
                    List<StrategyControlDetailList.DailyPlan.DailyPlanDetail> dailyPlanDetails = dailyPlan.getDailyPlanDetails();
                    StrategyDailyPlanInfoEntity dailyPlanInfoEntity = new StrategyDailyPlanInfoEntity();
                    dailyPlanInfoEntity.setCrossId(crossId);
                    dailyPlanInfoEntity.setDailyPlanId(dailyPlanId);
                    LambdaQueryWrapper<StrategyDailyPlanInfoEntity> delete = new LambdaQueryWrapper<>();
                    delete.eq(StrategyDailyPlanInfoEntity::getCrossId, crossId);
                    delete.eq(StrategyDailyPlanInfoEntity::getDailyPlanId, dailyPlanId);
                    strategyDailyPlanInfoMapper.delete(delete);
                    dailyPlanInfoEntity.setDailyPlanDetails(mapper.writeValueAsString(dailyPlanDetails));
                    strategyDailyPlanInfoMapper.insert(dailyPlanInfoEntity);
                }
            }
            StrategyControlVO strategyControlVO = convertDataList(list);
            StrategyControlReq req = convertReq(strategyControlVO);

            List<StrategyControlDataReq> data = req.getData();
            for (StrategyControlDataReq datum : data) {
                StrategyControlDataEntity strategyControlDataEntity = new StrategyControlDataEntity();
                strategyControlDataEntity.setBizId(datum.getBiz_id());
                strategyControlDataEntity.setBizType(datum.getBiz_type());
                strategyControlDataEntity.setStrategy(datum.getStrategy());
                SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                strategyControlDataEntity.setScheduleStart(simpleDateFormat.parse(datum.getSchedule_start()));
                strategyControlDataEntity.setScheduleStart(simpleDateFormat.parse(datum.getSchedule_start()));
                ObjectMapper instance = JacksonUtils.getInstance();
                strategyControlDataEntity.setTime(instance.writeValueAsString(datum.getTime_table()));
                strategyControlDataEntity.setFrequency(datum.getStrategy());
                strategyControlDataEntity.setStatus(datum.getStatus());
                strategyControlInfoMapper.insert(strategyControlDataEntity);
            }

            // 优化策略下发逻辑，保存是推送神思策略，下发时下发开关
            log.error("下发神思策略信息详情： {}", JacksonUtils.getInstance().writeValueAsString(req));
            Result result = pushStrategyControlService.push(req);
            if (Objects.nonNull(result)) {
                JsonViewObject jsonViewObject = JsonViewObject.newInstance();
                jsonViewObject.setCode(Integer.valueOf(result.getCode()));
                jsonViewObject.setMessage(result.getMsg());
                jsonViewObject.setContent(result.getData());
                jsonViewObject.setStatus(StringUtils.equals("200", result.getCode()) ? Constants.JsonView.STATUS_SUCCESS : Constants.JsonView.STATUS_FAIL);
                return jsonViewObject;
            } else {
                log.error("推送神思策略控制失败");
                return JsonViewObject.newInstance().fail("推送神思策略控制失败！");
            }
        } catch (Exception e) {
            log.error("策略管理数据保存, 推送神思远程服务失败：{}", e);
            throw new Exception(e);
        }
    }

    @Override
    public JsonViewObject strategyPush(StrategyControlDetailList list) throws Exception {
        List<StrategyControlDetailList.ExecutePlan> plans = list.getPlans();
        StrategyControlDetailList.ExecutePlan executePlan = plans.get(0);
        LambdaQueryWrapper<StrategyPlanInfoEntity> query = new LambdaQueryWrapper<>();
        query.eq(StrategyPlanInfoEntity::getPlanId, executePlan.getPlanId());
        query.eq(StrategyPlanInfoEntity::getCrossId, executePlan.getCrossId());
        strategyPlanInfoMapper.delete(query);
        StrategyPlanInfoEntity planInfoEntity = new StrategyPlanInfoEntity();
        planInfoEntity.setCrossId(executePlan.getCrossId());
        planInfoEntity.setPlanId(executePlan.getPlanId());
        planInfoEntity.setType(executePlan.getType());
        planInfoEntity.setCompany(executePlan.getCompany());
        planInfoEntity.setStatus(executePlan.getStatus());
        planInfoEntity.setStartTime(executePlan.getStartTime());
        planInfoEntity.setEndTime(executePlan.getEndTime());
        planInfoEntity.setPlanDetails(JacksonUtils.getInstance().writeValueAsString(executePlan.getPlanDetails()));
        strategyPlanInfoMapper.insert(planInfoEntity);
        StrategyControlVO strategyControlVO = convertDataList(list);
        StrategyControlReq req = convertReq(strategyControlVO);

        List<StrategyControlDataReq> data = req.getData();
        for (StrategyControlDataReq datum : data) {
            StrategyControlDataEntity strategyControlDataEntity = new StrategyControlDataEntity();
            strategyControlDataEntity.setBizId(datum.getBiz_id());
            strategyControlDataEntity.setBizType(datum.getBiz_type());
            strategyControlDataEntity.setStrategy(datum.getStrategy());
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            strategyControlDataEntity.setScheduleStart(simpleDateFormat.parse(datum.getSchedule_start()));
            strategyControlDataEntity.setScheduleStart(simpleDateFormat.parse(datum.getSchedule_start()));
            ObjectMapper instance = JacksonUtils.getInstance();
            strategyControlDataEntity.setTime(instance.writeValueAsString(datum.getTime_table()));
            strategyControlDataEntity.setFrequency(datum.getStrategy());
            strategyControlDataEntity.setStatus(datum.getStatus());
            strategyControlInfoMapper.insert(strategyControlDataEntity);
        }
        StrategyControlSwitchReq switchReq = getSwitchReq(req);
        Result witchPustResult = pushStrategyControlService.switchPush(switchReq);
        if (Objects.nonNull(witchPustResult)) {
            JsonViewObject jsonViewObject = JsonViewObject.newInstance();
            jsonViewObject.setCode(Integer.valueOf(witchPustResult.getCode()));
            jsonViewObject.setMessage(witchPustResult.getMsg());
            jsonViewObject.setStatus(StringUtils.equals("200", witchPustResult.getCode()) ? Constants.JsonView.STATUS_SUCCESS : Constants.JsonView.STATUS_FAIL);

            StringBuilder sb = new StringBuilder();
            List<LinkedHashMap<String, Object>>  contentList = (List<LinkedHashMap<String, Object>>) witchPustResult.getData();
            for (LinkedHashMap<String, Object> item : contentList) {
                Object result = item.get("result");
                if (Objects.equals(1, result)) {
                    sb.append(item.get("biz_id")).append(item.get("strategy")).append("策略控制成功");
                    jsonViewObject.setContent(sb);
                } else {
                    sb.append(item.get("biz_id")).append(item.get("strategy")).append("策略控制失败");
                    jsonViewObject.setContent(sb);
                }
            }

            return jsonViewObject;
        } else {
            log.error("推送神思策略控制开关失败");
            return JsonViewObject.newInstance().fail("推送神思策略控制开关失败！");
        }
    }

    private StrategyControlSwitchReq getSwitchReq(StrategyControlReq req) {
        StrategyControlSwitchReq switchReq = new StrategyControlSwitchReq();
        List<StrategyControlDataReq> data = req.getData();
        if (!CollectionUtils.isEmpty(data)) {
            List<StrategyControlSwitchReq.Detail> details = new ArrayList<>();
            for (StrategyControlDataReq datum : data) {
                StrategyControlSwitchReq.Detail detail = new StrategyControlSwitchReq.Detail();
                detail.setBiz_id(datum.getBiz_id());
                detail.setBiz_type(datum.getBiz_type());
                detail.setStrategy(datum.getStrategy());
                detail.setStatus(0);
                details.add(detail);
            }
            switchReq.setData(details);
        }
        return switchReq;
    }

    /**
     * 页面管理数据转化神思数据
     *
     * @param list
     */
    public StrategyControlVO convertDataList(StrategyControlDetailList list) throws Exception {
        try {
            if (Objects.nonNull(list)) {
                List<StrategyControlDetailList.DailyPlan> dailyPlans = list.getDailyPlans();
                List<StrategyPlanDTO> strategyPlanDTOS = new ArrayList<>();
                if (!CollectionUtils.isEmpty(dailyPlans)) {
                    for (StrategyControlDetailList.DailyPlan dailyPlan : dailyPlans) {
                        Integer dailyPlanId = dailyPlan.getDailyPlanId();
                        String crossId = dailyPlan.getCrossId();
                        List<StrategyControlDetailList.DailyPlan.DailyPlanDetail> dailyPlanDetails = dailyPlan.getDailyPlanDetails();

                        List<StrategyControlDetailList.DailyPlan.DailyPlanDetail> dailyPlanGroupList = new ArrayList<>(dailyPlanDetails.size());
                        Map<Integer, List<StrategyControlDetailList.DailyPlan.DailyPlanDetail>> dailyMap = dailyPlanDetails.stream().collect(Collectors.groupingBy(StrategyControlDetailList.DailyPlan.DailyPlanDetail::getStrategy));
                        for (Map.Entry<Integer, List<StrategyControlDetailList.DailyPlan.DailyPlanDetail>> entry : dailyMap.entrySet()) {
                            StrategyControlDetailList.DailyPlan.DailyPlanDetail result = new StrategyControlDetailList.DailyPlan.DailyPlanDetail();
                            Integer strategy = entry.getKey();
                            List<StrategyControlDetailList.DailyPlan.DailyPlanDetail> value = entry.getValue();
                            List<String> times = new ArrayList<>(value.size());
                            for (StrategyControlDetailList.DailyPlan.DailyPlanDetail dailyPlanDetail : value) {
                                List<String> timesList = dailyPlanDetail.getTimes();
                                String timeStr = timesList.get(0) + "-" + timesList.get(1);
                                times.add(timeStr);
                            }
                            result.setStrategy(strategy);
                            result.setTimes(times);
                            dailyPlanGroupList.add(result);
                        }

                        for (StrategyControlDetailList.DailyPlan.DailyPlanDetail dailyPlanDetail : dailyPlanGroupList) {
                            StrategyPlanDTO strategyPlanDTO = new StrategyPlanDTO();
                            Integer strategy = dailyPlanDetail.getStrategy();
                            strategyPlanDTO.setStrategy(strategy);
                            strategyPlanDTO.setTimes(dailyPlanDetail.getTimes());
                            strategyPlanDTO.setDailyPlanId(dailyPlanId);
                            strategyPlanDTO.setCrossId(crossId);
                            strategyPlanDTOS.add(strategyPlanDTO);
                        }
                    }
                }

                List<StrategyPlanDTO> resultList = new ArrayList<>();
                List<StrategyControlDetailList.ExecutePlan> plans = list.getPlans();
                if (!CollectionUtils.isEmpty(plans) && !CollectionUtils.isEmpty(dailyPlans)) {
                    for (StrategyControlDetailList.ExecutePlan plan : plans) {
                        Integer planId = plan.getPlanId();
                        Date startTime = plan.getStartTime();
                        Date endTime = plan.getEndTime();
                        Integer status = plan.getStatus();
                        List<StrategyControlDetailList.ExecutePlan.PlanDetail> planDetails = plan.getPlanDetails();
                        if (!CollectionUtils.isEmpty(planDetails)) {
                            for (StrategyControlDetailList.ExecutePlan.PlanDetail planDetail : planDetails) {
                                for (StrategyPlanDTO strategyPlanDTO : strategyPlanDTOS) {
                                    Integer dailyPlanId = planDetail.getDailyPlanId();
                                    List<Integer> weeks = planDetail.getWeeks();
                                    if (Objects.equals(dailyPlanId, strategyPlanDTO.getDailyPlanId())) {
                                        strategyPlanDTO.setPlanId(planId);
                                        strategyPlanDTO.setStartTime(startTime);
                                        strategyPlanDTO.setEndTime(endTime);
                                        strategyPlanDTO.setWeeks(weeks);
                                        strategyPlanDTO.setStatus(status);
                                        resultList.add(strategyPlanDTO);
                                    }
                                }
                            }
                        }
                    }
                }
                List<StrategyControlDataVO> dataList = new ArrayList<>();
                for (StrategyPlanDTO strategyPlanDTO : resultList) {
                    StrategyControlDataVO dataVO = new StrategyControlDataVO();
                    dataVO.setBizId(strategyPlanDTO.getCrossId());
                    dataVO.setBizType(0);
                    dataVO.setStrategy(strategyPlanDTO.getStrategy());
                    dataVO.setScheduleStart(strategyPlanDTO.getStartTime());
                    dataVO.setScheduleEnd(strategyPlanDTO.getEndTime());
                    dataVO.setAction("insert");
                    dataVO.setStatus(strategyPlanDTO.getStatus());
                    dataVO.setFrequency(strategyPlanDTO.getFreq());
                    List<StrategyControlDataVO.TimeTable> timeTables = new ArrayList<>();
                    List<Integer> weeks = strategyPlanDTO.getWeeks();
                    for (Integer week : weeks) {
                        StrategyControlDataVO.TimeTable timeTable = new StrategyControlDataVO.TimeTable();
                        timeTable.setWeek(week);
                        List<String> times = strategyPlanDTO.getTimes();
                        timeTable.setTimeList(times.toArray(new String[times.size()]));
                        timeTables.add(timeTable);
                    }
                    dataVO.setTime(timeTables);
                    dataList.add(dataVO);
                    StrategyControlVO req = new StrategyControlVO();
                    req.setDataList(dataList);
                    return req;
                }
            }
        } catch (Exception e) {
            log.error("数据转化神思数据结构异常：{}", e);
            throw new Exception(e);
        }
        return null;
    }

    @Override
    public JsonViewObject strategyFactoryList(StrategyFactoryQueryVO vo) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        LambdaQueryWrapper<StrategyFactoryEntity> queryWrapper = new LambdaQueryWrapper<>();
        if (Objects.isNull(vo)) {
            List<StrategyFactoryEntity> result = strategyFactoryMapper.selectList(queryWrapper);
            jsonViewObject.success(result);
        } else {
            Page<StrategyFactoryEntity> page = new Page<>(vo.getCurrentPage(), vo.getPageSize());
            if (Objects.nonNull(vo.getType())) {
                queryWrapper.eq(StrategyFactoryEntity::getType, vo.getType());
            }
            if (Objects.nonNull(vo.getScene())) {
                queryWrapper.eq(StrategyFactoryEntity::getScene, vo.getScene());
            }
            if (StringUtils.isNotBlank(vo.getStrategyName())) {
                queryWrapper.eq(StrategyFactoryEntity::getStrategyName, vo.getStrategyName());
            }
            if (StringUtils.isNotBlank(vo.getStrategyNo())) {
                queryWrapper.eq(StrategyFactoryEntity::getStrategyNo, vo.getStrategyNo());
            }
            if (StringUtils.isNotBlank(vo.getCompany())) {
                queryWrapper.eq(StrategyFactoryEntity::getCompany, vo.getCompany());
            }
            Page<StrategyFactoryEntity> result = strategyFactoryMapper.selectPage(page, queryWrapper);
            jsonViewObject.success(result);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject strategyFactorySave(StrategyFactoryEntity entity) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            if (Objects.nonNull(entity.getId())) {
                strategyFactoryMapper.updateById(entity);
            } else {
                strategyFactoryMapper.insert(entity);
            }
        } catch (Exception e) {
            log.error("策略库保存失败：{}", e);
            jsonViewObject.success("策略库保存失败");
        }
        return jsonViewObject.success("策略库保存成功");
    }

    @Override
    public JsonViewObject strategyFactoryDel(List<Integer> ids) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            if (!CollectionUtils.isEmpty(ids)) {
                strategyFactoryMapper.deleteBatchIds(ids);
            }
        } catch (Exception e) {
            log.error("策略库策略删除失败：{}", e);
            jsonViewObject.success("策略库策略删除失败");
        }
        return jsonViewObject.success("策略库删除成功");
    }
}
